/*                                     ___   ___               */
/*           ▄▄▄       ██▓     ██▓███  |\  \ |\  \             */
/*          ▒████▄    ▓██▒    ▓██░  ██▒\ \  \\_\  \            */
/*          ▒██  ▀█▄  ▒██░    ▓██░ ██▓▒ \ \______  \           */
/*          ░██▄▄▄▄██ ▒██░    ▒██▄█▓▒ ▒  \|_____|\  \          */
/*           ▓█   ▓██▒░██████▒▒██▒ ░  ░         \ \__\         */
/*           ▒▒   ▓▒█░░ ▒░▓  ░▒▓▒░ ░  ░          \|__|         */
/*            ▒   ▒▒ ░░ ░ ▒  ░░▒ ░                             */
/*         F a░a s▒e && ░S░c h░ä f e r  \                      */
/*                ░  ░    ░  ░            S o  2 0 1 9         */



/*▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄ INCLUDE ▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄*/
#include "world.h"
/*▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀ INCLUDE ▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀*/
  /*▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬▬*/



double min_size = 10;


/* myrand(0) --> coord */
/* myrand(1) --> mass  */
int64_t myrand(int mass)
{
  if (mass) { // mass has different max_value
    int64_t res = rand();
    if (rand() % 2)
    {
    res <<= 0x20;
    res += rand();
    }
    return ((rand() % (MAX_MASS / RAND_MAX) - 1) +1);
  }
  else {
    int64_t res = rand();
    if (rand() % 2)
    {
    res <<= 0x20;
    res += rand();
    }
    return (res % (WORLD_SIZE));
  }
}



/*▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄ IMPLEMENTATION ▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄▄*/
vec_2 get_force_vec(masspoint a, masspoint b)
{
  /* F * normalised vector = new OV */
  vec_2 v = minus(b.ov,a.ov);
  // vec_print(stdout, a.ov, "a.ov: ");
  // vec_print(stdout, b.ov, "b.ov: ");
  // vec_print(stdout, v, "get_force_vec v: ");
  uint64_t mag = magnitude(v);
  // printf("get_force_vec magn v: %"PRIu64"\n", mag);
  if (mag)
  {
    return scale_vec( v, ( b.m / pow(mag, 3) ));
  }
  return nullvector();
}



mp_list* init_world(parameter * param)
{

  double scale_rm = 0;
  MAX_MASS = INT64_MAX / (double) param->num_points;
  rm = nullvector();

  /* we need myrandom */
  srand(time(NULL));

  /* size_max = max value of vec_2 coordinates and mass */
  /* t_world ≤ size_max / 2 */

  /*▬▬▬▬▬▬▬▬▬▬▬ initialization of data structure ▬▬▬▬▬▬▬▬▬▬▬▬*/
   masspoints = calloc(param->num_points, sizeof(masspoint));

#ifdef WITH_OMP
#pragma omp for schedule (static)
#endif
  for (size_t i = 0; i < param->num_points; i++)
  {
    /* create myrandom coordinates and mass */
    vec_2 ov = {.x = myrand(0), .y = myrand(0)};
    uint64_t m = myrand(1);
    /* set starting vector */
    vec_2 fv = {.x = 0 , .y = 0};

    masspoints[i] = (masspoint) {.ov = ov, .m = m, .fv = fv, .centered = 0};
  }

mp_list* mpl = malloc(sizeof(mp_list));
mpl->mp = masspoints;
mpl->len = param->num_points;
   /* calculate ceter of mass */
#ifdef WITH_OMP
#pragma omp for schedule (static)
#endif
   for (size_t i = 0; i < param->num_points; i++)
   {
     scale_rm += (double) masspoints[i].m;
     rm = add(rm, scale_vec(masspoints[i].ov, masspoints[i].m));
   }
   scale_rm = 1 / scale_rm;
   rm = scale_vec(rm,scale_rm);
return mpl;
}



int step_world(mp_list* mplist)
{
  size_t num_points = mplist->len;
  masspoint * masspoints = mplist->mp;
  int activity = 0;
  double C = 1;
  /* calculate for all pairwise disjunct mass points the force vector */
  /* optimized for sequential workload */
#ifdef WITH_OMP
#pragma omp for schedule (static)
#endif
  for (size_t i = 0; i < num_points; i++)
  {
    /* clear the force vector beforehand */
    masspoints[i].fv = nullvector();
  }
  
  // /* calculate for all pairwise disjunct mass points the force vector */
  // /* optimized for sequential workload */
  //
#ifdef WITH_OMP
#pragma omp for schedule (static)
#endif
  for (size_t i = 0; i < num_points; i++)
  {
    // fprintf(stdout, "forcevector\n");
    if ( !masspoints[i].centered)
    {
      masspoint a = masspoints[i];
      activity = 1;
#ifdef WITH_OMP
#pragma omp for schedule (static)
#endif
      /* berechne for all masspoints the force vector */
      for (size_t j = 0; j < num_points; j++)
      {
        masspoint b = masspoints[j];

        /* and add them to a.v */
        a.fv = add(a.fv, get_force_vec(a,b));
      }

      /* masspoint "in" center of mass */
      if ( magnitude(minus(a.ov , rm)) <= min_size )
      {
        masspoints[i].ov = rm;
        masspoints[i].centered = 1;
        continue;
      }

      double scale_rm = 0;
      /* calculate center of mass */

#ifdef WITH_OMP
#pragma omp for schedule (static)
#endif
      for (size_t i = 0; i < num_points; i++)
      {
        scale_rm += (double) masspoints[i].m;
        rm = add(rm, scale_vec(masspoints[i].ov, masspoints[i].m));
      }
      scale_rm = 1 / scale_rm;
      rm = scale_vec(rm,scale_rm);

      /* INT64_MAX / 600 as in 10s for max movement */
      while ( 2000 >= magnitude(scale_vec(a.fv, C))
          && 65536 > C)
      {
        C *= 2;
        //printf("up C: %lf\n", C);
      }

      /* wenn fv linear abhänging zu rm */
      /* und |fv| > |rm| */
      /* und |rm| > δ */
      while ( magnitude(minus(rm, a.ov)) <= magnitude(scale_vec(a.fv, C)) )
      {
        C /= 2;
        //printf("down C: %lf\n", C);
      }

      a.fv = scale_vec(a.fv, C);
      masspoints[i].ov = add(a.ov, a.fv);
    }
  }



  // for (size_t i = 0; i < num_points; i++) {
  //   masspoints[i].ov = scale_vec(masspoints[i].ov, 0.5);
  // }

  return activity;
}

  /* sum them up */
  /* let the simulation run for t_world time */
/*▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀ IMPLEMENTATION ▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀▀*/
