/* xfont_Color.c */
#include "xgraph.h"
#include "stdint.h"
/* Funktionsprototypen */
static int create_window (parameter* param);
static void process_event (XEvent report);
static void eventloop (void);
static void close_window (void);
static int delay (int i);
static int time_diff (void);
static void draw_graphics (mp_list* mplist);
//static void print_text (GC, int, int, char *, char *);
static void setcolourvianame (GC, char *);
/* Globale Variablen */
static struct timeval st, rt;
static unsigned int width, height;
static Display *display;
static int screen;
static int depth;
static Window win;
char **my_fonts;
mp_list* mplist;
uint64_t scale;
uint64_t pause_flag = 1;


int ginit(parameter* param)
{
  int flag = 1;
  mplist = init_world(param);
  scale = WORLD_SIZE / (param->height - 5);
  create_window(param);
  int quit = 0;
  int pause = 0;
  int td;
  gettimeofday (&st, NULL);
  while (!quit) {
    gettimeofday (&rt, NULL);
    td = time_diff ();
    pause = delay (FRAME_LEN - td + pause);
    st = rt;
    // draw  60 frames per second
    eventloop ();
    draw_graphics(mplist);
    if ( flag && !pause_flag)
    {
      flag = step_world(mplist);
    }
  }
  return 0;
}

/* Funktion erzeugt ein Fenster */
static int create_window (parameter* param) {
  char *window_name = "nBody - Faase & Schaefer";
  char *icon_name   = "window";
  static XSizeHints size_hints;
  Window rootwin;
  width = param->width;
  height = param->height;


  /* X-Sitzung öffnen */
  display = XOpenDisplay (NULL);
  /* Fehlerüberprüfung */
  if (display == NULL) {
    printf ("Verbindung zum X-Server fehlgeschlagen?!?\n");
    exit(EXIT_FAILURE);
  }
  /* Standardbildschirm eines Displays */
  screen = XDefaultScreen (display);
  /* Standardtiefe des Screens - Anzahl der Ebenen */
  depth = XDefaultDepth (display, screen);
  rootwin = RootWindow (display, screen);
  win = XCreateSimpleWindow ( display, rootwin, 1, 1,
      width, height, 0,
      WhitePixel (display, screen),
      BlackPixel (display, screen) );

  size_hints.flags = PSize | PMinSize | PMaxSize;
  size_hints.min_width  = width;
  size_hints.max_width  = width;
  size_hints.min_height = height;
  size_hints.max_height = height;
  XSetStandardProperties ( display, win, window_name, icon_name,
      None, 0, 0, &size_hints);
  XSelectInput (display, win,
      ButtonPressMask | KeyPressMask | ExposureMask);
  XMapWindow (display, win);
  /* Schriftarten */

  int fcount;                /* Zahl der Fonts von XListFont  */
  my_fonts = XListFonts (display, "*", 5, &fcount);
  return 1;
}

static void close_window (void) {
  XCloseDisplay (display);
}

static void eventloop (void) {
  XEvent xev;
  int num_events;
  XFlush (display);
  num_events = XPending (display);
  while ((0 < num_events)) {
    num_events--;
    XNextEvent (display, &xev);
    process_event (xev);
  }
}


static void process_event (XEvent report) {
  KeySym key;
  switch (report.type) {
    case Expose:      /* Expose-Event --> Bild zeichnen */
      if (report.xexpose.count == 0) {

      }
      break;
    case KeyPress:
      key = XLookupKeysym (&report.xkey, 0);
      switch (key) {
        case (XK_Q):
        case (XK_q):
          free(mplist);
          close_window();
          break;
        case (XK_a):
          step_world(mplist);
          break;
        case (XK_p):
          pause_flag = (pause_flag) ? 0 : 1;
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
}

static void draw_graphics (mp_list* mplist) {
  /* unwrap struct mp_list */
   masspoint * masspoints = mplist->mp;
   size_t num_points = mplist->len;
  GC mygc;                      /* neu: Grafikkontext */
  unsigned long black, white;   /* Nummern der Farben */
  XMapRaised (display, win);
  /* Farbnummern besorgen */
  black = BlackPixel (display, screen);
  white = WhitePixel (display, screen);
  /* Graphical Context erzeugen, Farben eintragen */
  mygc = XCreateGC (display, win, 0, 0);
  XSetForeground (display, mygc, white);
  XSetBackground (display, mygc, black);
  /* sicherheitshalber alles löschen */
  XClearWindow (display, win);

  for (size_t i = 0; i< num_points; i++)
  {
    if ( masspoints[i].m <= ((MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "magenta");
    }
    if ( masspoints[i].m >= (((MAX_MASS / RAND_MAX) / 11) + 1) && masspoints[i].m <= (2 * (MAX_MASS / RAND_MAX) / 11) )
    {
      // setcolourvianame (mygc, "purple3");
    }
    if ( masspoints[i].m >= ((2 * (MAX_MASS / RAND_MAX) / 11) + 1) && masspoints[i].m <= (3 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "blue");
    }
    if ( masspoints[i].m >= (3 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= (4 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "DeepSkyBlue3");
    }
    if ( masspoints[i].m >= (4 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= (5 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "cyan");
    }
    if ( masspoints[i].m >= (5 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= (6 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "SpringGreen");
    }
    if ( masspoints[i].m >= (6 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= (7 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "forest green");
    }
    if ( masspoints[i].m >= (7 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= (8 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "chartreuse");
    }
    if ( masspoints[i].m >= (8 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= (9 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "yellow");
    }
    if ( masspoints[i].m >= (9 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= (10 * (MAX_MASS / RAND_MAX) / 11) )
    {
      setcolourvianame (mygc, "DarkOrange1");
    }
    if ( masspoints[i].m >= (10 * (MAX_MASS / RAND_MAX) / 11) + 1 && masspoints[i].m <= ((MAX_MASS / RAND_MAX) ) )
    {
      setcolourvianame (mygc, "red");
    }

   uint16_t x = masspoints[i].ov.x / scale;
   uint16_t y = masspoints[i].ov.y / scale;
  XFillArc (display, win, mygc, x, y, 5, 5, 0, 360 * 64);
  } // end of for

  XFlush (display);
  XFreeGC (display, mygc);
}


static void setcolourvianame (GC myGC, char *colourname) {
  /* genaue RGB-Komponenten der Farbe */
  XColor rgbcolour;
  /* am nächsten liegende RGB-Farben der Hardware */
  XColor hwarecolour;
  int status;
  /* Colormap */
  Colormap  myColourmap;
  int myDepth;
  myDepth = DefaultDepth (display, screen);
  myColourmap = DefaultColormap (display, screen);
  screen = DefaultScreen (display);
  if (myDepth > 1) {
    status = XLookupColor (
        display, myColourmap,colourname,&rgbcolour,&hwarecolour );
    if (status != 0) {
      /* Farben für die Colormap reservieren */
      status = XAllocColor ( display, myColourmap, &hwarecolour);
      if (status != 0) {
        /* Zeichenfarbe setzen */
        XSetForeground (display, myGC, hwarecolour.pixel );
      }
      else {
        printf ("Kann Farbzelle für %s nicht ermitteln\n",
            colourname);
        printf ("Verwende stattdessen Schwarz\n");
        XSetForeground(display, myGC,BlackPixel(display,screen));
      }
    }
    else {
      printf ("Kann Farbe %s nicht finden\n", colourname);
      printf ("Verwende stattdessen Schwarz\n");
      XSetForeground(display, myGC, BlackPixel(display, screen));
    }
  }
  else {
    printf("Keine Farbe in in dieser Tiefe:%d ?!\n", myDepth);
    printf ("Verwende stattdessen Schwarz \n");
    XSetForeground (display, myGC, BlackPixel(display, screen));
  }
}

static int delay(int i) {
  struct timeval timeout;
  if (i>0) {
    timeout.tv_usec = i % (unsigned long) 1000000;
    timeout.tv_sec = i / (unsigned long) 1000000;
    select(0, NULL, NULL, NULL, &timeout);
  }
  return (i>0 ? i : 0);
}
static int time_diff(void) {
  int diff;
  gettimeofday(&rt, NULL);
  diff = (1000000*(rt.tv_sec-st.tv_sec))+(rt.tv_usec-st.tv_usec);
  st = rt;
  return diff;
}
